@@ -49,15 +49,15 @@ struct heif_context
 
 struct heif_encoder
 {
-  heif_encoder(std::shared_ptr<heif::HeifContext> context,
-               const struct heif_encoder_plugin* plugin);
+  heif_encoder(const struct heif_encoder_plugin* plugin);
+
   ~heif_encoder();
 
   struct heif_error alloc();
+
   void release();
 
 
-  //std::shared_ptr<heif::HeifContext> context;
   const struct heif_encoder_plugin* plugin;
   void* encoder = nullptr;
 };
