@@ -31,10 +31,12 @@
 #include <memory>
 #include <cstring>
 #include <cassert>
+#include <string>
+#include <algorithm>
 
 #include <iostream>  // TODO: remove me
 
-#include "rav1e/rav1e.h"
+#include "rav1e.h"
 
 
 struct encoder_struct_rav1e
@@ -134,7 +136,7 @@ static void rav1e_init_parameters()
   p->type = heif_encoder_parameter_type_integer;
   p->integer.default_value = 4;
   p->has_default = true;
-  p->integer.have_minimum_maximum = true;
+  p->integer.have_minimum_maximum = false;
   p->integer.valid_values = valid_tile_num_values;
   p->integer.num_valid_values = 7;
   d[i++] = p++;
@@ -145,7 +147,7 @@ static void rav1e_init_parameters()
   p->type = heif_encoder_parameter_type_integer;
   p->integer.default_value = 4;
   p->has_default = true;
-  p->integer.have_minimum_maximum = true;
+  p->integer.have_minimum_maximum = false;
   p->integer.valid_values = valid_tile_num_values;
   p->integer.num_valid_values = 7;
   d[i++] = p++;
