@@ -277,6 +277,9 @@ bool HeifPixelImage::extend_to_size(int width, int height)
              &plane->mem[(plane->m_height - 1) * plane->stride],
              subsampled_width * nbytes);
     }
+
+    plane->m_width = subsampled_width;
+    plane->m_height = subsampled_height;
   }
 
   m_width = width;
